@@ -1,3 +1,4 @@
+use std::collections::HashMap;
 use std::fmt::Display;
 use std::marker::PhantomData;
 use std::rc::Rc;
@@ -108,7 +109,7 @@ impl AjarsWeb {
         base_url: P,
         interceptor: Rc<dyn Interceptor>,
     ) -> Result<AjarsWeb, Error> {
-        let window = window().ok_or_else(|| Error::MissingWindow)?;
+        let window = window().ok_or(Error::MissingWindow)?;
         Ok(AjarsWeb { window, interceptor, base_url: base_url.into() })
     }
 
@@ -118,20 +119,15 @@ impl AjarsWeb {
     ) -> RequestBuilder<'a, I, O, REST> {
         let url = format!("{}{}", &self.base_url, rest.path());
 
-        RequestBuilder {
-            rest,
-            window: &self.window,
-            interceptor: self.interceptor.as_ref(),
-            url,
-            phantom_i: PhantomData,
-            phantom_o: PhantomData,
-        }
+        RequestBuilder::new(rest, &self.window, url, self.interceptor.as_ref())
+            .add_header("Content-Type", "application/json")
     }
 }
 
 pub struct RequestBuilder<'a, I: Serialize + DeserializeOwned, O: Serialize + DeserializeOwned, REST: RestType<I, O>> {
     rest: &'a REST,
     interceptor: &'a dyn Interceptor,
+    headers: HashMap<String, String>,
     window: &'a Window,
     url: String,
     phantom_i: PhantomData<I>,
@@ -141,6 +137,44 @@ pub struct RequestBuilder<'a, I: Serialize + DeserializeOwned, O: Serialize + De
 impl<'a, I: Serialize + DeserializeOwned, O: Serialize + DeserializeOwned, REST: RestType<I, O>>
     RequestBuilder<'a, I, O, REST>
 {
+    pub fn new(rest: &'a REST, window: &'a Window, url: String, interceptor: &'a dyn Interceptor) -> Self {
+        RequestBuilder {
+            rest,
+            window,
+            interceptor,
+            url,
+            headers: HashMap::new(),
+            phantom_i: PhantomData,
+            phantom_o: PhantomData,
+        }
+    }
+
+    /// Add a header to the request
+    pub fn add_header<K: Into<String>, V: Into<String>>(mut self, key: K, value: V) -> Self {
+        self.headers.insert(key.into(), value.into());
+        self
+    }
+
+    /// Enable HTTP basic authentication.
+    pub fn basic_auth(self, username: &str, password: Option<&str>) -> Result<Self, Error> {
+        let user_pass = format!("{}:{}", username, password.unwrap_or_default());
+        let encoded_user_pass = self.window.btoa(&user_pass).map_err(|err| Error::Builder {
+            context: "Failed to encode in base64 the basic auth string".to_owned(),
+            source: err.into(),
+        })?;
+
+        Ok(self.add_header("AUTHORIZATION", format!("Basic {}", encoded_user_pass)))
+    }
+
+    /// Enable HTTP bearer authentication.
+    pub fn bearer_auth<T>(self, token: T) -> Self
+    where
+        T: std::fmt::Display,
+    {
+        let header_value = format!("Bearer {}", token);
+        self.add_header("AUTHORIZATION", header_value)
+    }
+
     /// Sends the Request to the target URL, returning a
     /// future Response.
     pub async fn send(self, data: &I) -> Result<O, Error> {
@@ -149,10 +183,12 @@ impl<'a, I: Serialize + DeserializeOwned, O: Serialize + DeserializeOwned, REST:
             source: WebError(format!("{:?}", err)),
         })?;
 
-        headers.append("Content-Type", "application/json").map_err(|err| Error::Builder {
-            context: "Failed to append Context-Type header".to_owned(),
-            source: WebError(format!("{:?}", err)),
-        })?;
+        for (header_key, header_value) in self.headers {
+            headers.append(&header_key, &header_value).map_err(|err| Error::Builder {
+                context: format!("Failed to append header with key [{}] and value [{}]", header_key, header_value),
+                source: WebError(format!("{:?}", err)),
+            })?;
+        }
 
         let mut opts = RequestInit::new();
         opts.mode(RequestMode::Cors);
@@ -182,7 +218,7 @@ impl<'a, I: Serialize + DeserializeOwned, O: Serialize + DeserializeOwned, REST:
             source: WebError(format!("{:?}", err)),
         })?;
 
-        let response = do_web_request(&self.window, request).await;
+        let response = do_web_request(self.window, request).await;
 
         let response = self.interceptor.after_response(response)?;
 
@@ -197,7 +233,7 @@ fn as_query_string<I: Serialize + DeserializeOwned>(
     data: &I,
 ) -> Result<(), Error> {
     opts.method(method);
-    uri.push_str("?");
+    uri.push('?');
     uri.push_str(&serde_urlencoded::to_string(data).map_err(|err| Error::Builder {
         context: "Failed to serialize data as query string".to_owned(),
         source: WebError(format!("{:?}", err)),
@@ -249,4 +285,3 @@ async fn into_http_response<O: Serialize + DeserializeOwned>(response: WebRespon
         Ok(data)
     }
 }
-
